@@ -106,6 +106,9 @@ static void gst_type_find_element_get_property (GObject * object,
 static const GstEventMask *gst_type_find_element_src_event_mask (GstPad * pad);
 static gboolean gst_type_find_element_src_event (GstPad * pad,
     GstEvent * event);
+static gboolean gst_type_find_handle_src_query (GstPad * pad,
+    GstQueryType type, GstFormat * fmt, gint64 * value);
+static void push_buffer_store (GstTypeFindElement * typefind);
 
 static void gst_type_find_element_chain (GstPad * sinkpad, GstData * data);
 static GstElementStateReturn
@@ -188,6 +191,8 @@ gst_type_find_element_init (GstTypeFindElement * typefind)
   gst_pad_set_event_function (typefind->src, gst_type_find_element_src_event);
   gst_pad_set_event_mask_function (typefind->src,
       gst_type_find_element_src_event_mask);
+  gst_pad_set_query_function (typefind->src,
+      GST_DEBUG_FUNCPTR (gst_type_find_handle_src_query));
   gst_pad_use_explicit_caps (typefind->src);
   gst_element_add_pad (GST_ELEMENT (typefind), typefind->src);
 
@@ -260,6 +265,34 @@ gst_type_find_element_get_property (GObject * object, guint prop_id,
       break;
   }
 }
+
+static gboolean
+gst_type_find_handle_src_query (GstPad * pad,
+    GstQueryType type, GstFormat * fmt, gint64 * value)
+{
+  GstTypeFindElement *typefind =
+      GST_TYPE_FIND_ELEMENT (gst_pad_get_parent (pad));
+  gboolean res;
+
+  res = gst_pad_query (GST_PAD_PEER (typefind->sink), type, fmt, value);
+  if (!res)
+    return FALSE;
+
+  if (type == GST_QUERY_POSITION && typefind->store != NULL) {
+    /* FIXME: this code assumes that there's no discont in the queue */
+    switch (*fmt) {
+      case GST_FORMAT_BYTES:
+        *value -= gst_buffer_store_get_size (typefind->store, 0);
+        break;
+      default:
+        /* FIXME */
+        break;
+    }
+  }
+
+  return TRUE;
+}
+
 static const GstEventMask *
 gst_type_find_element_src_event_mask (GstPad * pad)
 {
@@ -366,7 +399,8 @@ stop_typefinding (GstTypeFindElement * typefind)
       GST_WARNING_OBJECT (typefind,
           "could not seek to required position %u, hope for the best", size);
       typefind->mode = MODE_NORMAL;
-      gst_buffer_store_clear (typefind->store);
+      /* push out our queued buffers here */
+      push_buffer_store (typefind);
     } else {
       typefind->waiting_for_discont_offset = size;
     }
@@ -374,7 +408,7 @@ stop_typefinding (GstTypeFindElement * typefind)
 }
 
 static void
-push_buffer_store (GstTypeFindElement *typefind)
+push_buffer_store (GstTypeFindElement * typefind)
 {
   guint size = gst_buffer_store_get_size (typefind->store, 0);
   GstBuffer *buffer;
@@ -382,8 +416,7 @@ push_buffer_store (GstTypeFindElement *typefind)
   gst_pad_push (typefind->src, GST_DATA (gst_event_new_discontinuous (TRUE,
               GST_FORMAT_DEFAULT, (guint64) 0, GST_FORMAT_BYTES, (guint64) 0,
               GST_FORMAT_UNDEFINED)));
-  if (size
-      && (buffer = gst_buffer_store_get_buffer (typefind->store, 0, size))) {
+  if (size && (buffer = gst_buffer_store_get_buffer (typefind->store, 0, size))) {
     GST_DEBUG_OBJECT (typefind, "pushing cached data (%u bytes)", size);
     gst_pad_push (typefind->src, GST_DATA (buffer));
   } else {
@@ -433,6 +466,9 @@ gst_type_find_element_handle_event (GstPad * pad, GstEvent * event)
   TypeFindEntry *entry;
   GstTypeFindElement *typefind = GST_TYPE_FIND_ELEMENT (GST_PAD_PARENT (pad));
 
+  GST_DEBUG_OBJECT (typefind, "got event %d in mode %d", GST_EVENT_TYPE (event),
+      typefind->mode);
+
   switch (typefind->mode) {
     case MODE_TYPEFIND:
       switch (GST_EVENT_TYPE (event)) {
@@ -445,13 +481,15 @@ gst_type_find_element_handle_event (GstPad * pad, GstEvent * event)
             GST_INFO_OBJECT (typefind,
                 "'%s' is the best typefind left after we got all data, using it now (probability %u)",
                 GST_PLUGIN_FEATURE_NAME (entry->factory), entry->probability);
-            g_signal_emit (typefind, gst_type_find_element_signals[HAVE_TYPE], 0,
-                entry->probability, entry->caps);
+            g_signal_emit (typefind, gst_type_find_element_signals[HAVE_TYPE],
+                0, entry->probability, entry->caps);
             stop_typefinding (typefind);
+            push_buffer_store (typefind);
             gst_pad_event_default (pad, event);
           } else {
             gst_pad_event_default (pad, event);
-            GST_ELEMENT_ERROR (typefind, STREAM, TYPE_NOT_FOUND, (NULL), (NULL));
+            GST_ELEMENT_ERROR (typefind, STREAM, TYPE_NOT_FOUND, (NULL),
+                (NULL));
             stop_typefinding (typefind);
           }
           break;
@@ -476,6 +514,9 @@ gst_type_find_element_handle_event (GstPad * pad, GstEvent * event)
             gst_event_unref (event);
           }
         }
+      } else if (GST_EVENT_TYPE (event) == GST_EVENT_EOS) {
+        push_buffer_store (typefind);
+        gst_pad_event_default (pad, event);
       } else {
         gst_event_unref (event);
       }
