@@ -41,6 +41,24 @@ static GstStaticPadTemplate srctemplate = GST_STATIC_PAD_TEMPLATE ("src",
     GST_STATIC_CAPS_ANY);
 
 GST_DEBUG_CATEGORY_STATIC (queue_dataflow);
+#define GST_CAT_DEFAULT (queue_dataflow)
+
+#define STATUS(queue, msg) \
+  GST_CAT_LOG_OBJECT (queue_dataflow, queue, \
+		      "(%s:%s) " msg ": %u of %u-%u buffers, %u of %u-%u " \
+		      "bytes, %" G_GUINT64_FORMAT " of %" G_GUINT64_FORMAT \
+		      "-%" G_GUINT64_FORMAT " ns, %u elements", \
+		      GST_DEBUG_PAD_NAME (pad), \
+		      queue->cur_level.buffers, \
+		      queue->min_threshold.buffers, \
+		      queue->max_size.buffers, \
+		      queue->cur_level.bytes, \
+		      queue->min_threshold.bytes, \
+		      queue->max_size.bytes, \
+		      queue->cur_level.time, \
+		      queue->min_threshold.time, \
+		      queue->max_size.time, \
+		      queue->queue->length)
 
 static GstElementDetails gst_queue_details = GST_ELEMENT_DETAILS ("Queue",
     "Generic",
@@ -116,9 +134,13 @@ static void gst_queue_chain (GstPad * pad, GstData * data);
 static GstData *gst_queue_get (GstPad * pad);
 
 static gboolean gst_queue_handle_src_event (GstPad * pad, GstEvent * event);
+static gboolean gst_queue_handle_src_query (GstPad * pad,
+    GstQueryType type, GstFormat * fmt, gint64 * value);
 
 static GstCaps *gst_queue_getcaps (GstPad * pad);
-static GstPadLinkReturn gst_queue_link (GstPad * pad, const GstCaps * caps);
+static GstPadLinkReturn
+gst_queue_link_sink (GstPad * pad, const GstCaps * caps);
+static GstPadLinkReturn gst_queue_link_src (GstPad * pad, const GstCaps * caps);
 static void gst_queue_locked_flush (GstQueue * queue);
 
 static GstElementStateReturn gst_queue_change_state (GstElement * element);
@@ -286,7 +308,7 @@ gst_queue_init (GstQueue * queue)
       GST_DEBUG_FUNCPTR (gst_queue_chain));
   gst_element_add_pad (GST_ELEMENT (queue), queue->sinkpad);
   gst_pad_set_link_function (queue->sinkpad,
-      GST_DEBUG_FUNCPTR (gst_queue_link));
+      GST_DEBUG_FUNCPTR (gst_queue_link_sink));
   gst_pad_set_getcaps_function (queue->sinkpad,
       GST_DEBUG_FUNCPTR (gst_queue_getcaps));
   gst_pad_set_active (queue->sinkpad, TRUE);
@@ -296,11 +318,14 @@ gst_queue_init (GstQueue * queue)
       "src");
   gst_pad_set_get_function (queue->srcpad, GST_DEBUG_FUNCPTR (gst_queue_get));
   gst_element_add_pad (GST_ELEMENT (queue), queue->srcpad);
-  gst_pad_set_link_function (queue->srcpad, GST_DEBUG_FUNCPTR (gst_queue_link));
+  gst_pad_set_link_function (queue->srcpad,
+      GST_DEBUG_FUNCPTR (gst_queue_link_src));
   gst_pad_set_getcaps_function (queue->srcpad,
       GST_DEBUG_FUNCPTR (gst_queue_getcaps));
   gst_pad_set_event_function (queue->srcpad,
       GST_DEBUG_FUNCPTR (gst_queue_handle_src_event));
+  gst_pad_set_query_function (queue->srcpad,
+      GST_DEBUG_FUNCPTR (gst_queue_handle_src_query));
   gst_pad_set_active (queue->srcpad, TRUE);
 
   queue->cur_level.buffers = 0; /* no content */
@@ -370,7 +395,7 @@ gst_queue_getcaps (GstPad * pad)
 
   queue = GST_QUEUE (gst_pad_get_parent (pad));
 
-  if (queue->cur_level.bytes > 0) {
+  if (pad == queue->srcpad && queue->cur_level.bytes > 0) {
     return gst_caps_copy (queue->negotiated_caps);
   }
 
@@ -378,7 +403,50 @@ gst_queue_getcaps (GstPad * pad)
 }
 
 static GstPadLinkReturn
-gst_queue_link (GstPad * pad, const GstCaps * caps)
+gst_queue_link_sink (GstPad * pad, const GstCaps * caps)
+{
+  GstQueue *queue;
+  GstPadLinkReturn link_ret;
+
+  queue = GST_QUEUE (gst_pad_get_parent (pad));
+
+  if (queue->cur_level.bytes > 0) {
+    if (gst_caps_is_equal (caps, queue->negotiated_caps)) {
+      return GST_PAD_LINK_OK;
+    } else if (GST_STATE (queue) != GST_STATE_PLAYING) {
+      return GST_PAD_LINK_DELAYED;
+    }
+
+    /* Wait until the queue is empty before attempting the pad
+       negotiation. */
+    GST_QUEUE_MUTEX_LOCK;
+
+    STATUS (queue, "waiting for queue to get empty");
+    while (queue->cur_level.bytes > 0) {
+      g_cond_wait (queue->item_del, queue->qlock);
+      if (queue->interrupt) {
+        GST_QUEUE_MUTEX_UNLOCK;
+        return GST_PAD_LINK_DELAYED;
+      }
+    }
+    STATUS (queue, "queue is now empty");
+
+    GST_QUEUE_MUTEX_UNLOCK;
+  }
+
+  link_ret = gst_pad_proxy_pad_link (pad, caps);
+
+  if (GST_PAD_LINK_SUCCESSFUL (link_ret)) {
+    /* we store an extra copy of the negotiated caps, just in case
+     * the pads become unnegotiated while we have buffers */
+    gst_caps_replace (&queue->negotiated_caps, gst_caps_copy (caps));
+  }
+
+  return link_ret;
+}
+
+static GstPadLinkReturn
+gst_queue_link_src (GstPad * pad, const GstCaps * caps)
 {
   GstQueue *queue;
   GstPadLinkReturn link_ret;
@@ -461,23 +529,6 @@ gst_queue_handle_pending_events (GstQueue * queue)
   g_mutex_unlock (queue->event_lock);
 }
 
-#define STATUS(queue, msg) \
-  GST_CAT_LOG_OBJECT (queue_dataflow, queue, \
-		      "(%s:%s) " msg ": %u of %u-%u buffers, %u of %u-%u " \
-		      "bytes, %" G_GUINT64_FORMAT " of %" G_GUINT64_FORMAT \
-		      "-%" G_GUINT64_FORMAT " ns, %u elements", \
-		      GST_DEBUG_PAD_NAME (pad), \
-		      queue->cur_level.buffers, \
-		      queue->min_threshold.buffers, \
-		      queue->max_size.buffers, \
-		      queue->cur_level.bytes, \
-		      queue->min_threshold.bytes, \
-		      queue->max_size.bytes, \
-		      queue->cur_level.time, \
-		      queue->min_threshold.time, \
-		      queue->max_size.time, \
-		      queue->queue->length)
-
 static void
 gst_queue_chain (GstPad * pad, GstData * data)
 {
@@ -603,10 +654,12 @@ restart:
            * or its manager, switch back to iterator so bottom
            * half of state change executes */
           if (queue->interrupt) {
+            GstScheduler *sched;
+
             GST_CAT_DEBUG_OBJECT (queue_dataflow, queue, "interrupted");
             GST_QUEUE_MUTEX_UNLOCK;
-            if (gst_scheduler_interrupt (gst_pad_get_scheduler (queue->sinkpad),
-                    GST_ELEMENT (queue))) {
+            sched = gst_pad_get_scheduler (queue->sinkpad);
+            if (!sched || gst_scheduler_interrupt (sched, GST_ELEMENT (queue))) {
               goto out_unref;
             }
             /* if we got here because we were unlocked after a
@@ -727,10 +780,12 @@ restart:
        * manager, switch back to iterator so bottom half of state
        * change executes. */
       if (queue->interrupt) {
+        GstScheduler *sched;
+
         GST_CAT_DEBUG_OBJECT (queue_dataflow, queue, "interrupted");
         GST_QUEUE_MUTEX_UNLOCK;
-        if (gst_scheduler_interrupt (gst_pad_get_scheduler (queue->srcpad),
-                GST_ELEMENT (queue)))
+        sched = gst_pad_get_scheduler (queue->srcpad);
+        if (!sched || gst_scheduler_interrupt (sched, GST_ELEMENT (queue)))
           return GST_DATA (gst_event_new (GST_EVENT_INTERRUPT));
         goto restart;
       }
@@ -902,6 +957,37 @@ handled:
   return res;
 }
 
+static gboolean
+gst_queue_handle_src_query (GstPad * pad,
+    GstQueryType type, GstFormat * fmt, gint64 * value)
+{
+  GstQueue *queue = GST_QUEUE (gst_pad_get_parent (pad));
+  gboolean res;
+
+  if (!GST_PAD_PEER (queue->sinkpad))
+    return FALSE;
+  res = gst_pad_query (GST_PAD_PEER (queue->sinkpad), type, fmt, value);
+  if (!res)
+    return FALSE;
+
+  if (type == GST_QUERY_POSITION) {
+    /* FIXME: this code assumes that there's no discont in the queue */
+    switch (*fmt) {
+      case GST_FORMAT_BYTES:
+        *value -= queue->cur_level.bytes;
+        break;
+      case GST_FORMAT_TIME:
+        *value -= queue->cur_level.time;
+        break;
+      default:
+        /* FIXME */
+        break;
+    }
+  }
+
+  return TRUE;
+}
+
 static gboolean
 gst_queue_release_locks (GstElement * element)
 {
@@ -926,7 +1012,8 @@ gst_queue_change_state (GstElement * element)
 
   queue = GST_QUEUE (element);
 
-  GST_CAT_LOG_OBJECT (GST_CAT_STATES, element, "starting state change");
+  GST_CAT_LOG_OBJECT (GST_CAT_STATES, element,
+      "starting state change 0x%x", GST_STATE_TRANSITION (element));
 
   /* lock the queue so another thread (not in sync with this thread's state)
    * can't call this queue's _get (or whatever)
@@ -974,6 +1061,8 @@ gst_queue_change_state (GstElement * element)
       break;
   }
 
+  GST_QUEUE_MUTEX_UNLOCK;
+
   if (GST_ELEMENT_CLASS (parent_class)->change_state)
     ret = GST_ELEMENT_CLASS (parent_class)->change_state (element);
 
@@ -983,6 +1072,10 @@ gst_queue_change_state (GstElement * element)
   gst_pad_set_active (queue->sinkpad, TRUE);
   gst_pad_set_active (queue->srcpad, TRUE);
 
+  GST_CAT_LOG_OBJECT (GST_CAT_STATES, element, "done with state change");
+
+  return ret;
+
 unlock:
   GST_QUEUE_MUTEX_UNLOCK;
 
